@@ -1,6 +1,6 @@
 use 5.008007;
 package Test::ModuleVersion;
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 package
   Test::ModuleVersion::Object::Simple;
@@ -2567,42 +2567,25 @@ sub incr_reset {
     $self->{incr_parsing} = 0;
 }
 
-package Test::ModuleVersion;
+package
+  Test::ModuleVersion::ModuleURL;
 our @ISA = ('Test::ModuleVersion::Object::Simple');
 use strict;
 use warnings;
-use ExtUtils::Installed;
-use Carp 'croak';
-use Data::Dumper;
-
 sub has { __PACKAGE__->Test::ModuleVersion::Object::Simple::attr(@_) }
-has before => '';
+
 has distnames => sub { {} };
-has default_ignore => sub { ['Perl', 'Test::ModuleVersion'] };
-has ignore => sub { [] };
-has lib => sub { [] };
-has modules => sub { [] };
 has privates => sub { {} };
+has 'error';
+has lwp => 'auto';
 
-sub detect {
-  my $self = shift;
-  
-  # Detect installed modules
-  my $ei = ExtUtils::Installed->new;
-  my $modules = [];
-  push @$modules, [$_ => $ei->version($_)] for sort $ei->modules;
-  $self->modules($modules);
-  
-  return $self;
-}
-
-sub get_module_url {
+sub get {
   my ($self, $module, $version, $opts) = @_;
   
   $opts ||= {};
-  my $distnames = $opts->{distnames} || {};
-  my $privates = $opts->{privates} || {};
-  my $lwp = $opts->{lwp} || 'auto';
+  my $distnames = $self->distnames;
+  my $privates = $self->privates;
+  my $lwp = $self->lwp;
 
   # Module
   my $module_dist = $module;
@@ -2656,12 +2639,44 @@ sub get_module_url {
     else {
       $error = "Request to metaCPAN fail($res->{status_line}):$agent:$module_info";
     }
-    ${$opts->{error}} = $error if ref $opts->{error};
+    $self->error($error);
   }
   
   return $url;
 }
 
+
+package Test::ModuleVersion;
+our @ISA = ('Test::ModuleVersion::Object::Simple');
+use strict;
+use warnings;
+use ExtUtils::Installed;
+use Carp 'croak';
+use Data::Dumper;
+
+sub has { __PACKAGE__->Test::ModuleVersion::Object::Simple::attr(@_) }
+has before => '';
+has distnames => sub { {} };
+has default_ignore => sub { ['Perl', 'Test::ModuleVersion'] };
+has lib => sub { [] };
+has modules => sub { [] };
+has privates => sub { {} };
+
+sub detect {
+  my ($self, %opts) = @_;
+  my $ignore = $opts{ignore} || [];
+  
+  # Detect installed modules
+  my $ei = ExtUtils::Installed->new;
+  my @modules;
+  for my $module (sort $ei->modules) {
+    next if grep { $module eq $_ } @$ignore;
+    push @modules, [$module => $ei->version($module)];
+  }
+
+  return \@modules;
+}
+
 sub test_script {
   my ($self, %opts) = @_;
   
@@ -2728,8 +2743,6 @@ EOS
   my $test_count = 0;
   for my $m (@{$self->modules}) {
     my ($module, $version) = @$m;
-    next if grep { $module eq $_ } @{$self->default_ignore};
-    next if grep { $module eq $_ } @{$self->ignore};
     $code .= "  # $module\n"
       . "  \$require_ok = require_ok('$module');\n"
       . "  \$version_ok = is(\$${module}::VERSION, '$version', '$module version: $version');\n"
@@ -2752,11 +2765,13 @@ EOS
       : [];
     for my $m (@ms) {
       my ($module, $version) = @$m;
-      my $error;
-      my $url = $tm->get_module_url($module, $version,
-        {distnames => $distnames, privates => $privates, error => \$error, lwp => $lwp});
+      my $mu = Test::ModuleVersion::ModuleURL->new;
+      $mu->distnames($distnames);
+      $mu->privates($privates);
+      $mu->lwp($lwp);
+      my $url = $mu->get($module, $version);
       if (defined $url) { print "$url\n" }
-      else { print STDERR "$error\n" }
+      else { print STDERR $mu->error . "\n" }
     }  
   }
 }
@@ -2811,138 +2826,243 @@ sub _source {
 
 =head1 NAME
 
-Test::ModuleVersion - Module Version Test Generator (EXPERIMENTAL)
+Test::ModuleVersion - Module version test generator (EXPERIMENTAL)
 
 =head1 SYNOPSIS
 
   use Test::ModuleVersion;
-  
-  # Manually
   my $tm = Test::ModuleVersion->new;
   $tm->modules([
     ['DBIx::Custom' => '0.2108'],
     ['Validator::Custom' => '0.1426']
   ]);
-  print $tm->test_script;
+  $tm->test_script(output => 't/module.t');
 
-  # Automatically
-  my $tm = Test::ModuleVersion->new;
-  $tm->ignore([qw/Devel::NYTProf MySQL::Diff/]);
-  $tm->detect;
-  print $tm->test_script;
-  
 =head1 DESCRIPTION
 
-It is very difficult to install same modules in development
-environment and production environment.
+L<Test::ModuleVersion> is test generator for module version check.
+If you run the test generated by L<Test::ModuleVersion>,
+you can check the module version.
 
-C<cpan> is single versioning system, so you can't install
-specified version module well.
+If module version test is failed, you can list module URLs.
 
-Installation is very hard work if therer many modules.
-L<Test::ModuleVersion> help you.
+=head2 Create version test
 
-=head2 Create version checking test in C<development> environment.
-
-At first, you create test script in C<development> environment.
-
-L<Test::ModuleVersion> create version checking test manually
-or automatically
-
-This is C<manually> eamples.
+Let's create version test.
 
+  # mvt.pl
   my $tm = Test::ModuleVersion->new;
   $tm->modules([
     ['DBIx::Custom' => '0.2108'],
     ['Validator::Custom' => '0.1426']
   ]);
-  print $tm->test_script;
+  $tm->test_script(output => 't/module.t');
 
-You set C<modules> attribute which is list of module name and version number.
-You can get module test script by C<test_script> method.
+C<modules> attribute is set to the pairs of module and version.
+C<test_script> method print version test into C<t/module.t> file.
 
-run this script(name is C<mvt.pl>) to create test.
+Run C<mvt.pl>
 
- $ perl mvt.pl > t/module.t
+  $ perl mvt.pl
 
-the following-like test is created.
+Test script C<t/module.t> is created.
 
-    require_ok('DBIx::Custom');
-    module_version_is('DBIx::Custom', ExtUtils::Installed->version('DBIx::Custom'), '0.2108');
+  ...
+  $require_ok = require_ok('DBIx::Custom');
+  $version_ok = is($DBIx::Custom::VERSION, '0.2108', 'DBIx::Custom version: 0.2108');
 
-    require_ok('Validator::Custom');
-    module_version_is('Validator::Custom', ExtUtils::Installed->version('Validator::Custom'), '0.1426');
+  $require_ok = require_ok('Validator::Custom');
+  $version_ok = is($Validator::Custom::VERSION, '0.1426', 'DBIx::Custom version: 0.1426');
+  ...
 
-Or you can create module test C<automatically>.
+=head2 Run version test
 
-  my $tm = Test::ModuleVersion->new;
-  $tm->ignore([qw/Devel::NYTProf MySQL::Diff/]);
-  $tm->detect;
-  print $tm->test_script;
+Run version test.
 
-C<ignore> attribute is set to modules you want to ignore.
-C<detect> method detect all installed module and C<modules> attribute is set properly.
+  $ perl t/module.t
 
-=head2 Run test in C<production> environment.
-
-Second, the test script is moved to C<production> environment,
-and run the test.
-
-  perl t/module.t
-
-If the version in C<production> environment is different from C<development> one,
-test will fail.
+If module is not installed or version is different,
+test fail.
 
   ok 1 - require DBIx::Custom;
   not ok 2 - DBIx::Custom version: 0.2108
   #   Failed test 'DBIx::Custom version: 0.2108'
-  #   at module.t.pl line 13.
+  #   at t/module.t.pl line 13.
   #          got: '0.2106'
   #     expected: '0.2108'
 
   ok 2 - require Validator::Custom;
   ok 3 - Validator::Custom version: 0.1426
 
-It is very useful because you can know the module differnce.
+=head2 List module URLs
 
-=head2 Get module URLs
+You can list moudle URLs by C<list> command
 
-If test fail, you usually install the module manually, it is very hard work.
-you can get module URLs by C<list> command.
+  $ perl t/module.t list
 
-  $ perl module.t list
-
-URLs of all module in test sciprt is printed.
+All module URLs in version test is output to C<STDOUT>.
 
   http://cpan.metacpan.org/authors/id/K/KI/KIMOTO/DBIx-Custom-0.2108.tar.gz
   ...
 
-Internally, metaCPAN api is used to get module URL.
-if you have proxy server,
-you can set $ENV{http_proxy}.
+You can list only test failed module URLs by C<--fail> option
 
-You can also get test failed module URL by C<--fail> option.
+  $ perl t/module.t list --fail
 
-  $ perl module.t list --fail
+=head1 Advanced
 
-Module installation using L<cpanm> is very easy
-if cpanm is in current directory.
+=head2 Module installation by L<cpanm>
 
   $ perl t/module.t list --fail | perl cpanm -L extlib
 
-Modules is installed into C<extlib> directory.
+Module installation is very easy. Test failed module
+is installed into C<extlib> directory by L<cpanm>.
 
-If L<LWP::UserAgent> is available and version is 5.802+, test script use LWP::UserAgent
-for HTTP request.
-If L<LWP::UserAgent> is not available, test script use L<HTTP::Tiny>.
+=head2 HTTP client
 
-If you don't use L<LWP::UserAgent>, use C<--no-lwp> option.
+L<Test::Module> version switch two HTTP client as necessary.
 
-  $ perl module.t list --no-lwp
+=over 2
 
-If you force L<LWP::UserAgetn>, use C<--lwp> option.
+=item 1. LWP::UserAgent
 
-Have a fun to use L<Test::ModuleVersion>.
+=item 2. HTTP::Tiny
+
+=back
+
+These module is used to get module URLs from metaCPAN.
+
+If L<LWP::UserAgent> 5.802+ is installed, L<LWP::UserAgent>
+is seleced. If not, L<HTTP::Tiny> is selected.
+
+C<--lwp> option force L<LWP::UserAgent>.
+
+  $ perl t/module.t list --lwp
+
+C<--no-lwp> option force L<HTTP::Tiny>.
+
+  $ perl t/module.t list --no-lwp
+
+=head2 HTTP proxy
+
+  export http_proxy=http://hostname:3001
+
+C<http_proxy> environment variable enable you to use proxy server.
+
+=head2 HTTP proxy authentication
+
+  export http_proxy=http://username:password@hostname:3001
+
+If L<LWP::UserAgent> 5.802+ is installed,
+proxy authentication is available.
+L<HTTP::Tiny> don't support proxy authentication.
+
+=head1 EXAMPELS
+
+=head2 Basic1
+
+  # Directory
+  t / mvt.pl
+    / module.t
+  
+  extlib / lib / perl5 / Object / Simple.pm
+                       / Validator / Custom.pm
+
+features:
+
+=over 2
+
+=item 1. Module is installed in C<extlib/lib/perl5>
+
+=item 2. Perl 5.008007+ is required
+
+=item 3. Object::Simple 3.625, Validator::Custom 0.1401
+
+=back
+
+  use Test::ModuleVersion;
+  my $tm = Test::ModuleVersion->new;
+  $tm->lib('../extlib/lib/perl5');
+  $tm->before(<<'EOS');
+  use 5.008007;
+  
+  =pod
+
+  run mvt.pl to create this module version test(t/module.t).
+
+    perl mvt.pl
+  
+  =cut
+  EOS
+  $tm->modules([
+    ['Object::Simple' => '3.0625'],
+    ['Validator::Custom' => '0.1401']
+  ]);
+  $tm->test_script(output => 't/module.t');
+
+=head2 Basic2
+
+  # Directory
+  t / mvt.pl
+    / module.t
+  
+  extlib / lib / perl5 / LWP.pm
+
+features:
+
+=over 2
+
+=item 1. LWP 6.03
+
+LWP module distribution name is C<libwww-perl>.
+If module name is different from distribution name,
+you can use C<distnames> attribute.
+
+=back
+
+  use Test::ModuleVersion;
+  my $tm = Test::ModuleVersion->new;
+  $tm->lib('../extlib/lib/perl5');
+  $tm->distnames({
+    'LWP' => 'libwww-perl',
+  });
+  $tm->modules([
+    ['LWP' => '6.03'],
+  ]);
+  $tm->test_script(output => 't/module.t');
+  
+=head2 Basic3
+
+  # Directory
+  t / mvt.pl
+    / module.t
+  
+  extlib / lib / perl5 / SomeModule.pm
+
+features:
+
+=over 2
+
+=item 1. SomeModule 0.03 don't exist in CPAN
+
+=item 2. SomeModule exist in http://myhost/SomeModule-0.03.tar.gz
+
+SomeModule is private module.
+If module exist in some URL,
+you can use C<privates> attribute.
+
+=back
+
+  use Test::ModuleVersion;
+  my $tm = Test::ModuleVersion->new;
+  $tm->lib('../extlib/lib/perl5');
+  $tm->privates({
+    'SomeModule' => 'http://myhost/%M.tar.gz',
+  });
+  $tm->modules([
+    ['SomeModule' => '0.03'],
+  ]);
+  $tm->test_script(output => 't/module.t');
 
 =head1 ATTRIBUTES
 
@@ -2951,7 +3071,7 @@ Have a fun to use L<Test::ModuleVersion>.
   my $code = $self->before;
   $tm = $tm->before($code);
 
-You can add some code before test script.
+You can add some code before version test.
 
   $tm->before(<<'EOS');
   use 5.008007;
@@ -2980,136 +3100,67 @@ You can add some code before test script.
   });
 
 Module distribution name corresponding to module name.
-Some module is not same distribution name as module name,
-like LWP module.
+Some module have different distribution name.
+For example, L<LWP> module distribution name is C<libwww-perl>.
 
-LWP is module name, but distribution name is C<libwww-perl>.
-If right distribution name is unknown, L<Test::ModuleVersion>
-can't get module URL, so you must set C<distnames> attribute
-when distribution name is not same as module name.
+you must set C<distnames> attribute to get module URL.
 
 =head2 C<lib>
 
   my $lib = $self->lib;
-  $tm = $tm->lib('extlib/lib/perl5');
-  $tm = $tm->lib(['extlib/lib/perl5', ...]);
+  $tm = $tm->lib('../extlib/lib/perl5');
+  $tm = $tm->lib(['../extlib/lib/perl5', ...]);
 
-Module including pass from script directory.
-The following code is added to test script.
+Module including pass from version test directory.
+C<use lib> is added to version test.
 
-  use lib "$FindBin::Bin/extlib/lib/perl5";
-
-If the module is installed in this directory,
-module version test is success.
-
-=head2 C<ignore>
-
-  my $ignored_modules = $tm->ignore;
-  $tm = $tm->ignore([qw/Devel::NYTProf MySQL::Diff/]);
-
-ignored modules you don't want to contain in test script.
-
-=head2 C<default_ignore>
-
-  my $ignored_modules = $tm->ignore;
-  $tm = $tm->ignore(['Perl', 'Test::ModuleVersion']);
-
-Default ignored modules you don't want to contain in test script,
-default to C<['Perl']>.
-
-Don't use C<exculde_default> attribute usually.
-use C<ignore> attribute instead.
+  use lib "$FindBin::Bin/../extlib/lib/perl5";
 
 =head2 C<modules>
 
   my $modules = $tm->modules;
   $tm = $tm->modules($modules);
 
-List of Module name and version.
+Pairs of module and version.
 
   $tm->modules([
     ['DBIx::Custom' => '0.2108'],
     ['Validator::Custom' => '0.1426']
   ]);
 
-Version number must be string like C<'0.1426'>, not C<0.1426>.
-
-If C<detect> method is executed, C<modules> attribute is set automatically.
+Note that version must be string(C<'0.1426'>), not number(C<0.1426>).
 
 =head2 C<privates>
 
   my $privates = $tm->privates;
   $tm = $tm->privates({
-    'Some::Module' => 'http://localhost/~kimoto/%M.tar.gz'
+    'SomeModule' => 'http://localhost/~kimoto/%M.tar.gz'
   });
 
-Private repogitory URL. If you use C<privates> attribute,
-you don't need upload module to C<CPAN>.
-You upload module to some place you can access by C<http> protocal.
-
-C<%M> is replaces module name and version number, like Some-Module-0.01.
-You also use this attribute with C<distnames> attribute.
+Private module URLs.
+you can get module URL if the module don't exist in CPAN.
+C<%M> is replaced by C<module-version> like C<SomeModule-0.01>.
 
 =head1 METHODS
 
 =head2 C<detect>
 
-  $tm->detect;
+  my $modules = $tm->detect;
+  my $modules = $tm->detect(ignore => ['Perl', 'Test::ModuleVersion']);
 
-Detect all installed module and C<modules> attribute is set.
-
-=head2 C<get_module_url>
-
-  my $url = $tm->get_module_url($module, $version);
-  my $url = $tm->get_module_url($module, $version, $option);
-
-Get module URL by module name and version number.
-
-  # http://cpan.metacpan.org/authors/id/K/KI/KIMOTO/DBIx-Custom-0.2108.tar.gz
-  my $url = $tm->get_module_url('DBIx::Custom', '0.2108');
-
-You must specify version number as string, not number.
-for example, I<0.2110> is wrong, I<'0.2110'> is right.
-
-=over 2
-
-B<Option>
-
-You can set options
-
-  $tm->get_module_url($module, $version, {distnames => ..., ... => ...});
-
-=item * C<distnames>
-
-  distnames => {LWP => 'wwwlib-perl}
-
-=item * C<privates>
-
-  privates => {'Some::Module' => 'http://localhost/~kimoto/%M.tar.gz'}
-
-=item * C<error>
-
-  error => \$error
-
-You can get error message.
-
-=item * C<lwp>
-
-  lwp => 'use'
-  lwp => 'no'
-  lwp => 'auto'
-
-use L<LWP::UserAgent> or not or automatically use
-
-=back
+Get all installed module.
+If you set C<ignore> option, the module is ignored.
 
+Note that L<ExtUtils::Installed> is used internally.
+This information will be not accurate in some cases.
 
 =head2 C<test_script>
 
-  print $tm->test_script;
+  my $test_script = $tm->test_script;
+  $tm->test_script(output => 't/module.t');
 
-Test script which contains module version tests.
-C<modules> attirubtes is used to create test script.
+Return version test as string.
+If C<output> option is set, test is output to the file.
 
 =head1 AUTHOR
 
